@@ -45,6 +45,18 @@ def _copyfile(src: Path, dest: Path) -> None:
     shutil.copy(str(src), str(dest))
 
 
+def _copy_or_touch(src: Optional[Path], dest: Path) -> None:
+    """
+    Copy a single file, if specified, to a destination.
+
+    If the file src is not provided, create an empty file at dest.
+    """
+    if src:
+        _copyfile(src, dest)
+    else:
+        dest.touch()
+
+
 def setup_emulator(arch: str, output_dir: Path) -> None:
     """Copy the appropriate emulator binary to the output location."""
     emulator_name = 'qemu-{}-static'.format(arch)
@@ -52,7 +64,13 @@ def setup_emulator(arch: str, output_dir: Path) -> None:
     bin_dir.mkdir(parents=True, exist_ok=True)
     needs_emulator = (py_platform.system() != 'Darwin') and (py_platform.machine() != arch)
     if needs_emulator:
-        emulator_path = Path('/') / 'usr' / 'bin' / emulator_name
+        """
+        Using the same qemu binaries as the ones provided in
+        https://github.com/osrf/multiarch-docker-image-generation in order to
+        work around https://bugs.launchpad.net/qemu/+bug/1805913 and so qemu
+        supports renameat2() syscall.
+        """
+        emulator_path = Path(__file__).parent / 'qemu' / emulator_name
         if not emulator_path.is_file():
             raise RuntimeError('Could not find the expected QEmu emulator binary "{}"'.format(
                 emulator_path))
@@ -64,8 +82,10 @@ def setup_emulator(arch: str, output_dir: Path) -> None:
 def prepare_docker_build_environment(
     platform: Platform,
     ros_workspace: Path,
-    custom_setup_script: Optional[Path],
-    custom_data_dir: Optional[Path],
+    custom_setup_script: Optional[Path] = None,
+    custom_post_build_script: Optional[Path] = None,
+    colcon_defaults_file: Optional[Path] = None,
+    custom_data_dir: Optional[Path] = None,
 ) -> Path:
     """
     Prepare the directory for the sysroot.Docker build context.
@@ -73,6 +93,7 @@ def prepare_docker_build_environment(
     :param platform Information about the target platform
     :param ros_workspace Location of the ROS source workspace
     :param custom_setup_script Optional arbitrary script
+    :param colcon_defaults_file Optional colcon configuration file
     :param custom_data_dir Optional arbitrary directory for use by custom_setup_script
     :return The directory that was created.
     """
@@ -89,11 +110,17 @@ def prepare_docker_build_environment(
     else:
         custom_data_dest.mkdir(exist_ok=True)
 
-    custom_setup_dest = docker_build_dir / 'user-custom-setup'
-    if custom_setup_script:
-        _copyfile(custom_setup_script, custom_setup_dest)
+    _copy_or_touch(custom_setup_script, docker_build_dir / 'user-custom-setup')
+    _copy_or_touch(custom_post_build_script, docker_build_dir / 'user-custom-post-build')
+
+    if colcon_defaults_file:
+        _copyfile(colcon_defaults_file, docker_build_dir / 'defaults.yaml')
     else:
-        custom_setup_dest.touch()
+        (docker_build_dir / 'defaults.yaml').write_text("""
+build:
+  event-handlers: ["console_cohesion+","console_package_list+"]
+""")
+
     setup_emulator(platform.qemu_arch, docker_build_dir)
 
     return docker_build_dir
